var colors = require("colors")
var format = require("util").format
var inspect = require("util").inspect
var str = require("underscore.string")
var underscore = require("underscore")

var env = exports

env.has = function (name) {
  return name in process.env || (name + "_var") in process.env
}

env.get = function (name) {
  var pointer = name + "_var"

  if (name in process.env) {
    return process.env[name]
  } else if (pointer in process.env) {
    return env.get(process.env[pointer])
  } else {
    env.fail("Missing environment variable: %s", name)
  }
}

env.enabled = function (name) {
  return env.has(name) && !_(["", "0", "false"]).contains(env.get(name))
}

env.get.uint = function (name) {
  var value = env.get(name)

  if (/^\d+$/.test(value)) {
    return Number(value)
  } else {
    env.fail("Expected uint in $%s, got %s", name, inspect(value))
  }
}

env.get.url = function (name) {
  var value = env.get(name)

  try {
    return require("url").parse(value)
  } catch (error) {
    env.fail("Expected URL in $%s, got %s", name, inspect(value))
  }
}

env.eval = function (data) {
  data.replace(/\n$/, "").split("\n").forEach(function (line) {
    var match = line.match("(.*?)=(.*)")

    if (match) {
      process.env[match[1]] = match[2]
    } else {
      env.fail("Invalid env data:\n%s", stdout)
    }
  })
}

env.load = function (filename, callback) {
  var child_process = require("child_process")
  var path = require("path")

  if (path.existsSync(filename)) {
    child_process.exec(
      "source '" + filename + "' && env",
      function (error, stdout, stderr) {
        synchronously(function () {
          if (error && error.code) {
            env.fail(filename + ": " + stderr)
          } else if (error) {
            throw error
          } else {
            env.eval(stdout)
          }
        }, callback)
      }
    )
  } else {
    callback(null)
  }
}

env.resolve = function (raw_name) {
  var name = raw_name.replace(/_var$/, "")

  return name === raw_name ? [name] : [name].concat(
    env.resolve(process.env[raw_name])
  )
}

env.grep = function (filter) {
  return underscore.compact(
    Object.keys(process.env).sort().map(function (name) {
      if (filter.test(name)) {
        var chain = env.resolve(name)

        return format(
          "%s = %s", chain.join(" = "),
          colors.green(inspect(env.get(chain[0])))
        )
      }
    })
  ).join("\n")
}

env.raise = function () {
  var error = new Error(
    require("util").format.apply(null, arguments)
  )

  error.name = "ConfigError"
  error.type = "jungle"
  Error.captureStackTrace(error, env.error)

  throw error
}
